#include <fcntl.h>
#include <stdlib.h>
#include <unistd.h>
#include <netinet/in.h>
#include <string.h>
#include <stdio.h>

#include "gsm1.h"

int gsm_new(struct ast_trans_pvt *pvt)
{
	struct gsm_translator_pvt *tmp = (struct gsm_translator_pvt *)pvt->pvt;
	
	return (tmp->gsma = gsm_create()) ? 0 : -1;
}

void gsm_destroy_stuff(struct ast_trans_pvt *pvt)
{
	struct gsm_translator_pvt *tmp = (struct gsm_translator_pvt *)pvt->pvt;
	if (tmp->gsma)
		gsm_destroy(tmp->gsma);
}

/*! \brief decode and store in outbuf. */
int gsmtolin_framein(struct ast_trans_pvt *pvt, struct ast_frame *f)
{
	struct gsm_translator_pvt *tmp = (struct gsm_translator_pvt *)pvt->pvt;
	int x;
	int16_t *dst = (int16_t *)pvt->outbuf;
	/* guess format from frame len. 65 for MSGSM, 33 for regular GSM */
	int flen = (f->datalen % MSGSM_FRAME_LEN == 0) ?
		MSGSM_FRAME_LEN : GSM_FRAME_LEN;
	//printf("flen %i\n", flen);

	for (x=0; x < f->datalen; x += flen) {
		printf("gsm says: %i %i %i\n", x, f->datalen, flen);
		unsigned char data[2 * GSM_FRAME_LEN];
		unsigned char *src;
		int len;
		if (flen == MSGSM_FRAME_LEN) {
			len = 2*GSM_SAMPLES;
			src = data;
			/* Translate MSGSM format to Real GSM format before feeding in */
			/* XXX what's the point here! we should just work
			 * on the full format.
			 */
			//conv65(f->data + x, data);
			ast_log(LOG_WARNING, "We don't use conv65, sorry.\n");
		} else {
			len = GSM_SAMPLES;
			src = (unsigned char *)f->data + x;
		}
		/* XXX maybe we don't need to check */
		//printf("%i > %i ?\n", x, f->datalen);
		//printf("%i + %i > %i ?\n", pvt->samples, len, BUFFER_SAMPLES);
		if (pvt->samples + len > BUFFER_SAMPLES) {	
			ast_log(LOG_WARNING, "Out of buffer space\n");
			return -1;
		}
		if (gsm_decode(tmp->gsma, src, dst + pvt->samples)) {
			ast_log(LOG_WARNING, "Invalid GSM data (1)\n");
			return -1;
		}
		pvt->samples += GSM_SAMPLES;
		pvt->datalen += 2 * GSM_SAMPLES;
		if (flen == MSGSM_FRAME_LEN) {
			if (gsm_decode(tmp->gsma, data + GSM_FRAME_LEN, dst + pvt->samples)) {
				ast_log(LOG_WARNING, "Invalid GSM data (2)\n");
				return -1;
			}
			pvt->samples += GSM_SAMPLES;
			pvt->datalen += 2 * GSM_SAMPLES;
		}
	}
	return 0;
}


int audio_init(ao_device **device, char *init_card)
{
	ao_sample_format format;
	int default_driver;
	
	ao_initialize();
	default_driver = ao_default_driver_id();
	//default_driver = ao_driver_id("alsa09");
	//default_driver = ao_driver_id("alsa");
	
	format.bits = 16;
	format.channels = 2;
	format.rate = 8000;
	format.byte_format = AO_FMT_LITTLE;
	
	ao_option card;
	card.key="dev";
	card.value=init_card;
	card.next=0;
	ao_option *options = &card;
	
	*device = ao_open_live(default_driver, &format, options);
	if (*device == NULL) {
		fprintf(stderr, "Error opening device.\n");
		return 1;
	}
	return 0;
}

void audio_destroy(ao_device *device)
{
	ao_close(device);

	ao_shutdown();
}


int audio_play(char *buffer, int buf_size, ao_device *device)
{
	return ao_play(device, buffer, buf_size);
}

int file_read(char *filename, char *data, int maxlen)
{
	FILE *file;
	file = fopen(filename, "rb");
	if(file == 0) {
		perror("fopen");
		return 0;
	}
	//*data = (char *)malloc(maxlen);
	int ret = fread(data, 1, maxlen, file);
	fclose(file);
	return ret;
}

void mono_to_stereo(char *buffer, char *input, int input_size)
{
	int16_t *src = (int16_t *)input;
	int16_t *dst = (int16_t *)buffer;
	
	int i;
	for(i = 0; i < input_size; i++)
	{
		dst[2*i]   = src[i];
		dst[2*i+1] = src[i];
		//dst[4*i+2] = src[i];
		//dst[4*i+3] = src[i];
	}
}

#ifndef __GSM_LIBRARY__

#define FILE_BUFFER_SIZE 30000
int main(int argc, char **argv)
{
	// Usage: gsm1 [filename] [card]
	// It outputs via alsa with libao.
	char *filename;
	filename = "iaxd1b.gsm";
	if(argc >= 2)
	{
		filename = argv[1];
	}
	
	char *card;
	card = "default";
	if(argc >= 3)
	{
		card = argv[2];
	}
	
	struct gsm_translator_pvt gpvt;
	struct ast_trans_pvt pvt;
	memset(&pvt, 0, sizeof(struct ast_trans_pvt));
	pvt.pvt = &gpvt;
	if(gsm_new(&pvt) == -1) {
		printf("Error with gsm_new()\n");
		return -1;
	}
	struct ast_frame frame;
	frame.samples = 0;
	char all_data[FILE_BUFFER_SIZE];
	int all_datalen;
	all_datalen = file_read(filename, all_data, FILE_BUFFER_SIZE);
	int flen = (all_datalen % MSGSM_FRAME_LEN == 0) ?
			MSGSM_FRAME_LEN : GSM_FRAME_LEN;
	frame.datalen = flen * 10;
	frame.data = all_data;
	int curr_pos = 0;
	//printf("samples = %i, datalen = %i\n", frame.samples, frame.datalen);
	pvt.outbuf = malloc(1024*128);
	char *outbuf2c = malloc(1024*128*2);
	ao_device *device;
	if(audio_init(&device, card) != 0)
	{
		printf("audio init error.\n");
	}
	//FILE *outfile = fopen("out1.raw", "wb");
	//printf("outfile = %08x\n", outfile);
	while(curr_pos < all_datalen)
	{
		//printf("outbuf %08x, \n", pvt.outbuf);
		gsmtolin_framein(&pvt, &frame);
		//printf("samples = %i, datalen = %i, f.datalen = %i, curr_pos = %i\n", pvt.samples, pvt.datalen, frame.datalen, curr_pos);
		
		mono_to_stereo(outbuf2c, pvt.outbuf, pvt.samples*4);
		
		//fwrite(pvt.outbuf, pvt.samples, 1, outfile);
		if(audio_play(outbuf2c, pvt.samples*4, device) == 0)
		{
			printf("audio play error.\n");
		}
		//usleep(100000);
		curr_pos += frame.datalen;
		frame.data = all_data + curr_pos;
		pvt.samples = 0;
	}
	//fclose(outfile);
	audio_destroy(device);
	
	free(pvt.outbuf);
	gsm_destroy_stuff(&pvt);
	return 0;
}

#endif // __GSM_LIBRARY__
