/*
 * PyGSM
 * by Javantea aka. Joel R. Voss
 * 
 */

// I would very much rather people use pyalsa or something like that.
// AO audio skips when I use it of course.
//#define USE_AO_AUDIO


#include "pygsm3.h"

#define min(a, b) ((a) < (b)) ? (a) : (b)

pyGSMPacket *GetRxPacket(int *curr_pos, int all_datalen, 
	struct ast_trans_pvt *pvt, ao_device *device, struct ast_frame *frame);

static PyObject *decode(PyObject *self, PyObject *args)
{
	int curr_pos;
	struct gsm_translator_pvt gpvt;
	struct ast_trans_pvt pvt;
	struct ast_frame frame;
#ifdef USE_AO_AUDIO
	ao_device *device;
	char *card = "default";
#else
	void *device;
#endif // USE_AO_AUDIO
	
	char *data;
	int length;
	
	PyArg_ParseTuple(args, "s#", &data, &length);
	
	memset(&pvt, 0, sizeof(struct ast_trans_pvt));
	pvt.pvt = &gpvt;
	if(gsm_new(&pvt) == -1) {
		printf("Error with gsm_new()\n");
		Py_INCREF(Py_None);
		return Py_None;
	}
	frame.samples = 0;
	//all_datalen = file_read(filename, all_data, 30000);
	int flen = (length % MSGSM_FRAME_LEN == 0) ?
			MSGSM_FRAME_LEN : GSM_FRAME_LEN;
	frame.datalen = min(length, flen * 10);
	frame.data = data;
	//printf("samples = %i, datalen = %i\n", frame.samples, frame.datalen);
	pvt.outbuf = (char *)malloc(1024*128);
	curr_pos = 0;
	
#ifdef USE_AO_AUDIO
	if(audio_init(&device, card) != 0)
	{
		printf("audio init error.\n");
	}
#endif // USE_AO_AUDIO
	
	printf("Data len = %i. flen = %i.\n", length, flen);
	
	PyObject *ret = 0;
	
	while(curr_pos < length) {
		pyGSMPacket *sound = GetRxPacket(&curr_pos, length, &pvt, device, &frame);
		if (sound == 0)
		{
			break;
			//Py_INCREF(Py_None);
			//return Py_None;
		}
		
		// FIXME: this loop needs to append to ret, not overwrite.
		ret = Py_BuildValue("s#", sound->data, pvt.samples*4);
		//printf("samples*4 = %i, curr_pos = %i\n", pvt.samples*4, curr_pos);
		pvt.samples = 0;
	}
	
	free(pvt.outbuf);
	//free(all_data);
	gsm_destroy_stuff(&pvt);
#ifdef USE_AO_AUDIO
	audio_destroy(device);
#endif // USE_AO_AUDIO
	
	return ret;
}

pyGSMPacket *GetRxPacket(int *curr_pos, int all_datalen, 
	struct ast_trans_pvt *pvt, ao_device *device, struct ast_frame *frame)
{
	if(*curr_pos >= all_datalen)
	{
		printf("No more packets! %i >= %i\n", *curr_pos, all_datalen);
		return 0;
	}
	
	pyGSMPacket *sound = (pyGSMPacket *)malloc(sizeof(pyGSMPacket));
	
	gsmtolin_framein(pvt, frame);
	
	mono_to_stereo(sound->data, pvt->outbuf, pvt->samples*4);
	
	//fwrite(pvt.outbuf, pvt.samples, 1, outfile);
#ifdef USE_AO_AUDIO
	if(audio_play(sound->data, pvt->samples*4, device) == 0)
	{
		printf("audio play error.\n");
	}
#endif // USE_AO_AUDIO
	
	*curr_pos += frame->datalen;
	frame->data += frame->datalen;
	
	
	return sound;
}


static PyMethodDef module_methods[] = {
	{"decode", decode, METH_VARARGS, "Decode a gsm packets."},
	{NULL}  /* Sentinel */
};

#ifndef PyMODINIT_FUNC	/* declarations for DLL import/export */
#define PyMODINIT_FUNC void
#endif
PyMODINIT_FUNC initpygsm3(void)
{
    PyObject* m;

    m = Py_InitModule3("pygsm3", module_methods,
                       "Module to decode gsm.");

    if (m == NULL)
      return;
}
